@@ -28,13 +28,15 @@
 #include <gst/audio/streamvolume.h>
 #include <gst/audio/audio-format.h>
 
+bool EngineGstPipeline::s_previousMute = false;
+
 EngineGstPipeline::EngineGstPipeline()
     : m_currentSinkElement(nullptr)
     , m_replayGainElement(nullptr)
     , m_equalizerElement(nullptr)
-    , m_analyzerDataSize(512)
     , m_channels(1)
     , m_handlingAboutToFinish(false)
+    , m_currentState(GST_STATE_NULL)
     , m_tickTimer(new QTimer(this))
     , m_waitingForNextSource(true)
     , m_waitingForPreviousSource(false)
@@ -49,10 +51,9 @@ EngineGstPipeline::EngineGstPipeline()
 
     DEBUG_BLOCK
     qRegisterMetaType<GstState>("GstState");
-    m_pipeline = GST_PIPELINE(gst_element_factory_make("playbin", nullptr));
+    m_pipeline = GST_PIPELINE(gst_element_factory_make("playbin3", nullptr));
     gst_object_ref_sink (m_pipeline);
-    g_signal_connect(m_pipeline, "audio-tags-changed", G_CALLBACK(cb_audioTagsChanged), this);
-    g_signal_connect(m_pipeline, "notify::source", G_CALLBACK(cb_setupSource), this);
+    g_signal_connect(m_pipeline, "source-setup", G_CALLBACK(cb_setupSource), this);
     g_signal_connect(m_pipeline, "about-to-finish", G_CALLBACK(cb_aboutToFinish), this);
 
     GstBus *bus = gst_pipeline_get_bus(m_pipeline);
@@ -168,10 +169,12 @@ EngineGstPipeline::EngineGstPipeline()
         gst_object_unref(pad);
 
         g_object_set( GST_BIN(m_pipeline), "audio-sink", bin, nullptr);
+        s_previousMute = isMuted();
     }
     else
     {
         debug() << "failed to create custom playback bin";
+        s_previousMute = isMuted();
         g_signal_connect( m_pipeline, "notify::volume", G_CALLBACK (cb_volumeChanged), this);
         g_signal_connect( m_pipeline, "notify::mute", G_CALLBACK (cb_muteChanged), this);
     }
@@ -509,26 +512,22 @@ EngineGstPipeline::cb_duration(GstBus *bus, GstMessage *gstMessage, gpointer dat
 }
 
 void
-EngineGstPipeline::cb_setupSource(GstElement *playbin, GParamSpec *param, gpointer data)
+EngineGstPipeline::cb_setupSource(GstElement *playbin, GstElement *source, gpointer data)
 {
     Q_UNUSED(playbin);
-    Q_UNUSED(param);
     DEBUG_BLOCK;
 
-    GstElement *src;
     EngineGstPipeline *that = static_cast<EngineGstPipeline*>(data);
     Q_ASSERT(that->m_pipeline);
-    Q_ASSERT(G_IS_OBJECT(that->m_pipeline));
-    g_object_get(that->m_pipeline, "source", &src, nullptr);
 
     if ( ( that->currentSource().scheme().startsWith(QLatin1String("http") ) ||
            that->currentSource().scheme().startsWith(QLatin1String("rt") ) ) // rtp, rtsp
         // Check whether this property exists.
         // Setting it on a source other than souphttpsrc (which supports it) may break playback.
-        && g_object_class_find_property(G_OBJECT_GET_CLASS(src), "user-agent") )
+        && g_object_class_find_property(G_OBJECT_GET_CLASS(source), "user-agent") )
     {
         QString userAgent = ( QStringLiteral( "Amarok/" ) + QStringLiteral(AMAROK_VERSION) );
-        g_object_set(src, "user-agent", userAgent.toUtf8().constData(), nullptr);
+        g_object_set(source, "user-agent", userAgent.toUtf8().constData(), nullptr);
     }
 }
 
@@ -549,19 +548,22 @@ EngineGstPipeline::cb_muteChanged(GstElement *playbin, GParamSpec *spec, gpointe
     Q_UNUSED(playbin)
 
     EngineGstPipeline *that = static_cast<EngineGstPipeline *>(data);
-    Q_EMIT that->mutedChanged( that->isMuted() );
+    bool state = that->isMuted();
+    // Due to way how gstreamer is used (abused?), this callback might be fired multiple times
+    // for a single mute state, with isMuted() returning a value different from the expected one
+    // during first callback. This would trigger a loop in overlying code.
+    // One path of fixing would be altering how gstreamer is used, offloading the signals
+    // (and mute state queries) to main thread with g_idle_add or so.
+    // However, here is the "moral imperative" solution for this method:
+    // EngineGstPipeline should never claim that mute state has changed if it actually hasn't.
+    // Let's enforce that.
+    if( state != s_previousMute )
+        Q_EMIT that->mutedChanged( state );
+    s_previousMute = state;
     Q_UNUSED(spec)
     Q_UNUSED(data)
 }
 
-void
-EngineGstPipeline::cb_audioTagsChanged(GstElement *playbin, gint stream, gpointer data)
-{
-    Q_UNUSED(playbin)
-    EngineGstPipeline *that = static_cast<EngineGstPipeline *>(data);
-    Q_EMIT that->audioTagChanged(stream);
-}
-
 gboolean
 EngineGstPipeline::cb_buffering(GstBus *bus, GstMessage *gstMessage, gpointer data)
 {
@@ -1168,8 +1170,7 @@ inline void EngineGstPipeline::analyzerConvertAndEmit(bool isEndOfMedia)
 
 
     for (int j = 0 ; j < m_channels ; ++j) {
-        // QVector::resize doesn't reallocate the buffer
-        m_analyzerChannelBuffers[j].resize(0);
+        m_analyzerChannelBuffers[j].clear();
     }
 }
 
@@ -1196,11 +1197,7 @@ void EngineGstPipeline::analyzerProcessBuffer(GstElement*, GstBuffer* buffer, Gs
     // TODO emit endOfMedia
     EngineGstPipeline *that = static_cast<EngineGstPipeline *>(gThat);
 
-    // Copied locally to avoid multithead problems
-    qint32 dataSize = that->m_analyzerDataSize;
-    if (dataSize == 0) {
-        return;
-    }
+    const qint32 dataSize = 512; // was m_dataSize in original phonon-gstreamer, hard-coded here to default 512
 
     int channelsCount = 0;
 
